@@ -17,10 +17,16 @@ import math
 from itertools import islice, izip, tee
 from blockdiag.utils.myitertools import istep
 from blockdiag.utils.PILTextFolder import PILTextFolder as TextFolder
-import Image
-import ImageDraw
-import ImageFont
-import ImageFilter
+try:
+    from PIL import Image
+    from PIL import ImageDraw
+    from PIL import ImageFont
+    from PIL import ImageFilter
+except ImportError:
+    import Image
+    import ImageDraw
+    import ImageFont
+    import ImageFilter
 from blockdiag.utils.XY import XY
 from blockdiag.utils import ellipse
 from blockdiag.utils import urlutil
@@ -77,7 +83,9 @@ def dashize_line(line, length):
             yield (p1, p2)
 
 
-class ImageDrawEx(ImageDraw.ImageDraw):
+class ImageDrawEx(object):
+    self_generative_methods = []
+
     def __init__(self, filename, size, **kwargs):
         if kwargs.get('im'):
             self.image = kwargs.get('im')
@@ -87,20 +95,18 @@ class ImageDrawEx(ImageDraw.ImageDraw):
         self.filename = filename
         self.scale_ratio = kwargs.get('scale_ratio', 1)
         self.mode = kwargs.get('mode')
-        ImageDraw.ImageDraw.__init__(self, self.image, self.mode)
+        self.draw = ImageDraw.ImageDraw(self.image, self.mode)
 
     def resizeCanvas(self, size):
         image = self.image.resize(size, Image.ANTIALIAS)
-        return ImageDrawEx(self.filename, None, im=image,
-                           scale_ratio=self.scale_ratio)
+        self.image = self.image.resize(size, Image.ANTIALIAS)
+        self.draw = ImageDraw.ImageDraw(self.image, self.mode)
 
     def smoothCanvas(self):
-        image = self.image
         for i in range(15):
-            image = image.filter(ImageFilter.SMOOTH_MORE)
+            self.image = self.image.filter(ImageFilter.SMOOTH_MORE)
 
-        return ImageDrawEx(self.filename, None, im=image,
-                           scale_ratio=self.scale_ratio)
+        self.draw = ImageDraw.ImageDraw(self.image, self.mode)
 
     def arc(self, box, start, end, **kwargs):
         style = kwargs.get('style')
@@ -117,9 +123,9 @@ class ImageDrawEx(ImageDraw.ImageDraw):
                 end += 360
 
             for pt in ellipse.dots(box, length, start, end):
-                ImageDraw.ImageDraw.line(self, [pt, pt], fill=kwargs['fill'])
+                self.draw.line([pt, pt], fill=kwargs['fill'])
         else:
-            ImageDraw.ImageDraw.arc(self, box, start, end, **kwargs)
+            self.draw.arc(box, start, end, **kwargs)
 
     def ellipse(self, box, **kwargs):
         if 'filter' in kwargs:
@@ -134,7 +140,7 @@ class ImageDrawEx(ImageDraw.ImageDraw):
                 kwargs2 = dict(kwargs)
                 if 'outline' in kwargs2:
                     del kwargs2['outline']
-                ImageDraw.ImageDraw.ellipse(self, box, **kwargs2)
+                self.draw.ellipse(box, **kwargs2)
 
             if 'outline' in kwargs:
                 kwargs['fill'] = kwargs['outline']
@@ -146,16 +152,18 @@ class ImageDrawEx(ImageDraw.ImageDraw):
                 length = 4
 
             for pt in ellipse.dots(box, length):
-                ImageDraw.ImageDraw.line(self, [pt, pt], fill=kwargs['fill'])
+                self.draw.line([pt, pt], fill=kwargs['fill'])
         else:
             if kwargs.get('fill') == 'none':
                 del kwargs['fill']
 
-            ImageDraw.ImageDraw.ellipse(self, box, **kwargs)
+            self.draw.ellipse(box, **kwargs)
 
     def line(self, xy, **kwargs):
-        style = kwargs.get('style')
+        if 'jump' in kwargs:
+            del kwargs['jump']
 
+        style = kwargs.get('style')
         if kwargs.get('fill') == 'none':
             pass
         elif style in ('dotted', 'dashed'):
@@ -164,7 +172,7 @@ class ImageDrawEx(ImageDraw.ImageDraw):
             if 'style' in kwargs:
                 del kwargs['style']
 
-            ImageDraw.ImageDraw.line(self, xy, **kwargs)
+            self.draw.line(xy, **kwargs)
 
     def dashed_line(self, xy, **kwargs):
         style = kwargs.get('style')
@@ -191,7 +199,7 @@ class ImageDrawEx(ImageDraw.ImageDraw):
             d = int(math.ceil(thick / 2.0))
 
         if fill and fill != 'none':
-            ImageDraw.ImageDraw.rectangle(self, box, fill=fill)
+            self.draw.rectangle(box, fill=fill)
 
         x1, y1, x2, y2 = box
         lines = (((x1, y1), (x2, y1)), ((x1, y2), (x2, y2)),  # horizonal
@@ -212,7 +220,7 @@ class ImageDrawEx(ImageDraw.ImageDraw):
                 del kwargs2['style']
             if 'outline' in kwargs2:
                 del kwargs2['outline']
-            ImageDraw.ImageDraw.polygon(self, xy, **kwargs2)
+            self.draw.polygon(xy, **kwargs2)
 
         if kwargs.get('outline'):
             kwargs['fill'] = kwargs['outline']
@@ -237,7 +245,7 @@ class ImageDrawEx(ImageDraw.ImageDraw):
 
         if ttfont is None:
             if self.scale_ratio == 1:
-                ImageDraw.ImageDraw.text(self, xy, string, fill=fill)
+                self.draw.text(xy, string, fill=fill)
             else:
                 size = self.textsize(string)
                 image = Image.new('RGBA', size)
@@ -251,7 +259,7 @@ class ImageDrawEx(ImageDraw.ImageDraw):
 
                 self.image.paste(text_image, xy, text_image)
         else:
-            size = self.textsize(string, font=ttfont)
+            size = self.draw.textsize(string, font=ttfont)
 
             # Generate mask to support BDF(bitmap font)
             mask = Image.new('1', size)
@@ -262,7 +270,7 @@ class ImageDrawEx(ImageDraw.ImageDraw):
             filler = Image.new('RGB', size, fill)
             self.image.paste(filler, xy, mask)
 
-            ImageDraw.ImageDraw.__init__(self, self.image, self.mode)
+            self.draw = ImageDraw.ImageDraw(self.image, self.mode)
 
     def textarea(self, box, string, **kwargs):
         lines = TextFolder(box, string, scale=self.scale_ratio, **kwargs)
@@ -303,11 +311,26 @@ class ImageDrawEx(ImageDraw.ImageDraw):
             y = box[1]
 
         self.image.paste(image, (x, y))
-        ImageDraw.ImageDraw.__init__(self, self.image, self.mode)
+        self.draw = ImageDraw.ImageDraw(self.image, self.mode)
 
     def save(self, filename, size, format):
         if filename:
             self.filename = filename
 
+        if size is None:
+            x = int(self.image.size[0] / self.scale_ratio)
+            y = int(self.image.size[1] / self.scale_ratio)
+            size = (x, y)
+
         self.image.thumbnail(size, Image.ANTIALIAS)
-        self.image.save(self.filename, format)
+
+        if self.filename:
+            self.image.save(self.filename, format)
+            image = None
+        else:
+            import cStringIO
+            tmp = cStringIO.StringIO()
+            self.image.save(tmp, format)
+            image = tmp.getvalue()
+
+        return image
